<?php
 defined('ABSPATH') or die('No script kiddies please!');
 
 /*
 Plugin Name: Consent Manager
 Description: Consent Manager is the easiest way to reach the new privacy standard.
 Version: 2.0
 Author: consentmanager
 Author URI: https://www.consentmanager.de
 License: GPL2
 */
 
 require_once dirname(__FILE__) . '/ConsentManagerMain.php';
 
 $id = '';
 $mode = 3;
 $ignoreDomains = '';
 try
 {
  $id = esc_attr(get_option(consentManagerMain::OPTION_ID));
  $mode = esc_attr(get_option(consentManagerMain::OPTION_MODE));
  $ignoreDomains = esc_attr(get_option(consentManagerMain::OPTION_IGNORE_DOMAINS));
 }
 catch(Exception $e)
 {
 }
 
 //Actions______________________________________________________________________________________________
 
 if(function_exists('amp_is_request') && amp_is_request())
 {
  //AMP Code
  add_action('wp_head', 'amp_script');
  add_action('after_body_open_tag', 'amp_body');
 }
 else
  if((int)$mode === 1){
  add_action( 'wp_enqueue_scripts', 'consent_manager_scripts' );
 }
 else
 {
  if((int)$mode === 2)
  {
   //Serverside blocking
   add_action('after_setup_theme', 'buffer_start');
   add_action('shutdown', 'buffer_end');
  }
  //add_action('wp_enqueue_scripts', 'consent_manager_style');
  add_action('wp_footer', 'consent_manager_inline_scripts');
 }
 
 /**
  * inject automatic script for clientside mode
  */
 function consent_manager_scripts()
 {
  echo '<script type="text/javascript" data-cmp-ab="1" src="https://'. ConsentManagerMain::CDN .'/delivery/automatic.min.js" data-cmp-id="'.esc_attr(get_option(consentManagerMain::OPTION_ID)).'" data-cmp-host="'. ConsentManagerMain::HOST .'" data-cmp-cdn="'. ConsentManagerMain::CDN .'"></script>';
 }
 
 /**
  * Inject CM Widget script
  */
 function consent_manager_inline_scripts()
 {
  
  $id = '';
  $mode = 3;
  try
  {
   $id = esc_attr(get_option(consentManagerMain::OPTION_ID));
   $mode = esc_attr(get_option(consentManagerMain::OPTION_MODE));
  }
  catch(Exception $e)
  {
  }
  
  if((int)$id > 0)
  {
   if(function_exists('amp_is_request') && amp_is_request())
   {
    //AMP Code
    add_action('wp_head', 'amp_script');
    add_action('after_body_open_tag', 'amp_body');
   }
   else
   {
    //serverside automatic & clientside semi automatic
    if((int)$mode != 1)
    {
     consent_manager_style();
     echo ConsentManagerMain::getHTML((int)esc_attr(get_option(consentManagerMain::OPTION_ID)));
    }
   }
  }
 }
 
 /**
  * Register CM Widget style
  */
 function consent_manager_style()
 {
  wp_register_style('consentmanagerstyle', 'https://'. ConsentManagerMain::CDN .'/delivery/cmp.min.css', '', '1.1.0', 'all');
  
  wp_enqueue_style('consentmanagerstyle');
 }
 
 //Serverside blocking________________________________________________________________________________________
 
 /**
  * manipulate output buffer to block JS serverside
  * @param $buffer
  * @return array|string|string[]
  */
 function callbackForBuffer($buffer)
 {
  $id = '';
  $ignoreDomains = '';
  try
  {
   $id = esc_attr(get_option(consentManagerMain::OPTION_ID));
   $ignoreDomains = esc_attr(get_option(consentManagerMain::OPTION_IGNORE_DOMAINS));
  }
  catch(Exception $e)
  {
  }
  
  $_buffer = $buffer;
  //caching -> 1Tag
  $cache_file =  $id . '.js';
  $url = 'https://'. ConsentManagerMain::HOST .'/delivery/customblocking/' . $id . '.js';
  
  if (file_exists($cache_file) && (filemtime($cache_file) > (time() - 60 * 60 * 24 ))) {
   // Cache file is less than five minutes old.
   // Don't bother refreshing, just use the file as-is.
   $file = file_get_contents($cache_file);
  } else {
   // Our cache is out-of-date, so load the data from our remote server,
   // and also save it over our cache for next time.
   $file = file_get_contents($url);
   file_put_contents($cache_file, $file, LOCK_EX);
  }
  
  //scripts
  $blocking = json_decode($file);
  preg_match_all('#<script(.*?)</script>#is', $_buffer, $matches);
  foreach($matches[0] as $value)
  {
   $_v = $value;
   $src = getUrlFromString($value);
   if($src != '')
   {
    if(strpos($_v, 'class="') !== false)
    {
     $_v = str_replace('class="', 'class="cmplazyload ', $_v);
    }
    else
    {
     $_v = str_replace("<script", '<script class="cmplazyload"', $_v);
    }
    $domain = getDomainFromUrl($src);
    foreach($blocking as $key => $b)
    {
     if(strpos($domain, $b->d) !== false && strpos($ignoreDomains, $b->d) === false)
     {
      $r = '<script data-cmp-vendor="' . $b->v . '"';
      $_v = str_replace("<script", $r, $_v);
      $_v = str_replace("src", "data-cmp-src", $_v);
      break;
     }
     if($key === count($blocking)){
      sendUnblockedDomainInfo($domain);
     }
    }
   }
   else
   {
    //Nothing to do when no src attribute is detected in script tag
   }
   // modify buffer here, and then return the updated code
   $_buffer = str_replace($value, $_v, $_buffer);
  }
  
  //iframes
  preg_match_all('#<iframe(.*?)</iframe>#is', $_buffer, $matches);
  foreach($matches[0] as $value)
  {
   $_v = $value;
   $src = getUrlFromString($value);
   if($src != '')
   {
    if(strpos($_v, 'class="') !== false)
    {
     $_v = str_replace('class="', 'class="cmplazyload ', $_v);
    }
    else
    {
     $_v = str_replace("<iframe", '<iframe class="cmplazyload"', $_v);
    }
    $domain = getDomainFromUrl($src);
    foreach($blocking as $key => $b)
    {
     if(strpos($domain, $b->d) !== false && strpos($ignoreDomains, $b->d) === false)
     {
      $r = '<iframe data-cmp-vendor="' . $b->v . '"';
      $_v = str_replace("<iframe", $r, $_v);
      $_v = str_replace("src", "data-cmp-src", $_v);
      break;
     }
     if($key === count($blocking)){
      sendUnblockedDomainInfo($domain);
     }
    }
   }
   else
   {
    //Nothing to do when no src attribute is detected in script tag
   }
   // modify buffer here, and then return the updated code
   $_buffer = str_replace($value, $_v, $_buffer);
  }
  return $_buffer;
 }
 
 /**
  * get current site url
  * @param false $protocol
  * @return mixed|string
  */
 function getSiteURL($protocol = false)
 {
  $_protocol = ((!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
  $domainName = $_SERVER['HTTP_HOST'];
  return $protocol ? $_protocol . $domainName : $domainName;
 }
 
 /**
  * send vendor info to cm
  * @param $domain
  * @return false|string
  */
 function sendUnblockedDomainInfo($domain)
 {
  if(rand (1,100) === 1){
   file_get_contents('https://'. ConsentManagerMain::HOST .'/delivery/alertdomains.php?id=' . esc_attr(get_option(consentManagerMain::OPTION_ID)) . '&ref=' . urlencode(getSiteUrl()) . '&url=' . urlencode($domain));
  }
  return true;
 }
 
 /**
  * get src content from string
  * @param $string
  * @return mixed|string
  */
 function getUrlFromString($string)
 {
  preg_match('/(src=["\'](.*?)["\'])/', $string, $match);  //find src="X" or src='X'
  $split = preg_split('/["\']/', $match[0]); // split by quotes
  $src = $split[1]; // X between quotes
  return $src;
 }
 
 /**
  * get host from url
  * @param $url
  * @return mixed|string
  */
 function getDomainFromUrl($url)
 {
  if($url !== '')
  {
   $info = parse_url($url);
   $host = $info['host'];
   return $host;
  }
  return false;
 }
 
 /**
  * Check when HTML is completely build
  */
 function buffer_start()
 {
  ob_start("callbackForBuffer");
 }
 
 /**
  * Flush Output stream
  */
 function buffer_end()
 {
  if(ob_get_length())
  {
   ob_end_flush();
  }
 }
 
 //Shortcodes__________________________________________________________________________________________________
 
 /**
  * Shortcode function to get vendorlist
  */
 function wp_consentmanager_vendorlist_shortcode()
 {
  echo '<div id="cmpvendorlist"></div>';
  wp_register_script('consentmanagervendorsscript', 'https://consentmanager.net/delivery/vendorlist.php?id=1', '', '1.0', true);
  wp_enqueue_script('consentmanagervendorsscript');
 }
 
 /**
  * Shortcode function to get cookielist
  */
 function wp_consentmanager_cookielist_shortcode()
 {
  echo '<div id="cmpcookieinfo"></div>';
  wp_register_script('consentmanagercookiesscript', 'https://consentmanager.net/delivery/cookieinfo.php?id=1', '', '1.0', true);
  wp_enqueue_script('consentmanagercookiesscript');
 }
 
 add_shortcode('consentmanager_vendors', 'wp_consentmanager_vendorlist_shortcode');
 add_shortcode('consentmanager_cookies', 'wp_consentmanager_cookielist_shortcode');
 
 //Admin stuff_______________________________________________________________________________________
 
 add_action('admin_menu', 'consentManagerMenu');
 
 /** Add Settings button to plugin overview page */
 if(!function_exists('consentManager_admin_action_links'))
 {
  /**
   * Add Buttons and links to Admin page
   * @param $links
   * @param $file
   * @return mixed|void
   */
  function consentManager_admin_action_links($links, $file)
  {
   if(!current_user_can('edit_posts'))
   {
    return;
   }
   static $my_plugin;
   if(!$my_plugin)
   {
    $my_plugin = plugin_basename(__FILE__);
   }
   if($file == $my_plugin)
   {
    $settings_link = "<a href='/wp-admin/admin.php?page=ConsentManagerMain::ADMIN_URL'>Settings</a>";
    array_unshift($links, $settings_link);
   }
   return $links;
  }
  
  /**
   * Add left main menu item
   */
  function consentManagerMenu()
  {
   if(!current_user_can('edit_posts'))
   {
    return;
   }
   
   $icon = null; //icon for menu
   add_menu_page('Consent Manager Options', 'Consent Manager', 'manage_options', 'ConsentManagerMain::ADMIN_URL', 'consentManagerDisplayAdminPage', '/wp-content/plugins/ConsentManager/assets/images/logo20.png', 80);
  }
  
  /**
   * Display view
   */
  function consentManagerDisplayAdminPage()
  {
   if(!current_user_can('edit_posts'))
   {
    return;
   }
   if(!current_user_can('manage_options'))
   {
    wp_die(__('You do not have sufficient permissions to access this page.'));
   }
   include_once 'views/admin.php';
  }
  
 }
 
 add_filter('plugin_action_links', 'consentManager_admin_action_links', 10, 2);
 
 /**
  * Style for Admin settings
  */
 function admin_register_head()
 {
  $siteurl = get_option('siteurl');
  $url = $siteurl . '/wp-content/plugins/' . basename(dirname(__FILE__)) . '/assets/css/admin.css';
  echo "<link rel='stylesheet' type='text/css' href='$url' />\n";
 }
 
 add_action('admin_head', 'admin_register_head');
 
 //AMP Functions______________________________________________________________________________________
 
 /**
  * AMP Head injection
  */
 function amp_script()
 {
  ?>
  <script async custom-element="amp-consent" src="https://cdn.ampproject.org/v0/amp-consent-0.1.js"></script>
  <meta name="amp-consent-blocking" content="amp-ad">
  <?php
 }
 
 /**
  * AMP Bodx injection
  */
 function amp_body()
 {
  ?>
  <amp-consent id="ConsentManager" layout="nodisplay" type="ConsentManager">
   <script type="application/json">
    {
     "postPromptUI": "postPromptUI",
     "clientConfig": {
      "id": "<?php echo esc_attr(get_option(consentManagerMain::OPTION_ID)) ?>",
      "params": ""
     }
    }
   </script>
   <div id="postPromptUI">
    <button on="tap:ConsentManager.prompt()" role="button">Manage privacy settings</button>
   </div>
  </amp-consent>
  <?php
 }
 
 //Helpers____________________________________________________________________________________________
 
 if(!function_exists('dump'))
 {
  /**
   * debugging function
   * @param $var
   */
  function dump($var)
  {
   echo "<pre><div align='left'>";
   print_r($var);
   echo "</div></pre>";
  }
 }
